#ifndef __ACTIVYLCD_CPP
#define __ACTIVYLCD_CPP

//#include "alcd.h"
#include "activylcd.h"

#undef LCDDEBUG
#undef D_EVENTFILTER

///////////////////////////////////////////////////
// activy300 LCD plugin for VDR
//
// Original written by:
//
// Meinrad Sauter <meinrad.sauter@gmx.de>
// andreas 'randy' weinberger <vdr@smue.org>
//
// rearranged, beautified and rewritten by:
//
// Markus Geisler <geisler@primusnetz.de>
//

/////////////////////////////////////////////////////////
// Konstruktor

activyLCD::activyLCD()
{
	char output[30];

#ifdef LCDDEBUG
	logfile = fopen("/tmp/debuglog.alcd","w");
#endif
	locked = 0;
	powerpressed = 0;
	errorcounter = 0;

	// get current time
	error_currenttime = time(NULL);

	// set keycounter to null
	keycounter = NULL;

	SerialPortOpen();
  	LCDInit();  
  	SetPowerButton(0);

  	snprintf(output,2, "%s", HWREV);
  	SendCommandToLCD(output,2 );
  	snprintf(output,2, "%s", FWREV);
  	SendCommandToLCD(output,2 );
}

/////////////////////////////////////////////////////////
// Destruktor

activyLCD::~activyLCD()
{
  	LCDClear();
  	SetPowerButton(1);
  	SetLEDBrightness(LEDBrightnessOnExit);
  	SerialPortClose();
#ifdef LCDDEBUG
	fclose(logfile);
#endif
}

/////////////////////////////////////////////////////////
// Open Serial Port

int activyLCD::SerialPortOpen(void)
{
	struct termios options;

	// open serial device
  	fd = open(LCD_PORT, O_RDWR | O_NOCTTY | O_NDELAY);

	// check result
  	if (fd == -1) 
	{
		// error - port could not be opened
		return(-1);
  	} 
	else 
	{
		// ok - port opened
    		fcntl(fd, F_SETFL, 0);
	  	// Get the current options for the port...
		tcgetattr(fd, &options);
	  	// Set the baud rates to 38400...
	  	cfsetispeed(&options, B38400);
	  	cfsetospeed(&options, B38400);
	  	// Enable the receiver and set local mode...
	  	options.c_cflag |= (CLOCAL);
	  	options.c_cflag &= ~PARENB;
	  	options.c_cflag &= ~CSTOPB;
	  	options.c_cflag &= ~CSIZE;
	  	options.c_cflag |= CS8;
	  	options.c_oflag &= ~OPOST;
                options.c_oflag &= ~(ICANON | ECHO | ECHOE | ISIG);
                options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
		options.c_cc[VMIN]  = 0;                  /* warten auf min. 0 Zeichen */
		options.c_cc[VTIME] = 10;                 /* Timeout 10 Sekunden */

	  	// Set the new options for the port...
	  	tcsetattr(fd, TCSANOW, &options);

	  	return (fd);
  	}
}

/////////////////////////////////////////////////////////
// Close Serial Port

void activyLCD::SerialPortClose()
{
  	close(fd);
}

/////////////////////////////////////////////////////////
// Send Command to LCD-Device

void activyLCD::SendCommandToLCD(char *command,int length)
{
	char eventergebnis1;
	char eventergebnis2;
	char ergebnis1;
	char ergebnis2;
	char ergebnis3;
	char ergebnis4;
	char ergebnis5;
	char ergebnis6;
	char output[30];
	int result;

	if(command && length && !locked)
	{
                locked = 1;
	 	//SQ: enter critical area
	 	semop(semid, &enter, 1);
		//isyslog("alcd-plugin: entering critical area");
	
		//isyslog("alcd-plugin: following command will be send to lcd: |%s|\n",command);

#ifdef D_EVENTFILTER
  		write(fd,EVENTFILTER,2);
              	read(fd,&eventergebnis1,1);
               	read(fd,&eventergebnis2,1);

		// eventfilter set?
		if((eventergebnis1 == '\x95') && (eventergebnis2 == '\xA0'))
		{
			// eventfilter has been set!
		}
		else
		{
			isyslog("alcd-plugin: ERROR: during eventfilter set\n");
		}
#endif

//		// isyslog("alcd-plugin: following command will be send to lcd: |%s|\n",command);

		// send real command to lcd
	  	write(fd,command,length);
                result = read(fd,&ergebnis1,1);
                result = read(fd,&ergebnis2,1);

		// ergebniswerte auswerten
		switch (ergebnis1)
		{
			// blast ir buffer
			case '\x85' : 
			{
				// blast ir buffer
				if(ergebnis2 == '\xA0')
				{
					// ir buffer has been blasted!
				}
				else
				{
					errorcounter++;
					isyslog("alcd-plugin: ERROR: during blast\n");
				}
				break;
			}
			// hw revision
			case '\x89' : 
			{
				// hw revision - 1 byte!
				if(ergebnis2 == '\xA0')
				{
	                		read(fd,&ergebnis3,1);
					isyslog("alcd-plugin: Activy300-LCD-HW Revision: %i\n",ergebnis3);
				}
				else
				{
					errorcounter++;
					isyslog("alcd-plugin: Activy300-LCD-HW Revision could not be determined\n");
				}
				break;
			}
			// fw revision
			case '\x8a' : 
			{
				// fw revision - 4 bytes!
				if(ergebnis2 == '\xA0')
				{
	                		read(fd,&ergebnis3,1);
			                read(fd,&ergebnis4,1);
			                read(fd,&ergebnis5,1);
			                read(fd,&ergebnis6,1);

					isyslog("alcd-plugin: Activy300-LCD-SW Revision: Release: %i Version: %i PreRelease: %c VirtualNumber: %i\n",ergebnis3,ergebnis4,ergebnis5,ergebnis6);
				}
				else
				{
					errorcounter++;
					isyslog("alcd-plugin: Activy300-LCD-SW Revision could not be determined\n");
				}
				break;
			}
			// set LED 
			case '\x90' : 
			{
				// set LED
				if(ergebnis2 == '\xA0')
				{
					// LED has been set!
				}
				else
				{
					errorcounter++;
					isyslog("alcd-plugin: ERROR: during set LED\n");
				}
				break;
			}
			// set powerbutton
			case '\x94' : 
			{
				// set powerbutton
				if(ergebnis2 == '\xA0')
				{
					// powerbutton has been set!
				}
				else
				{
					errorcounter++;
					isyslog("alcd-plugin: ERROR: during powerbutton set\n");
				}
				break;
			}
			// clear lcd
			case '\x9b' : 
			{
				// clear lcd
				if(ergebnis2 == '\xA0')
				{
					// lcd has been cleaned!
				}
				else
				{
					errorcounter++;
					isyslog("alcd-plugin: ERROR: during lcd clear\n");
				}
				break;
			}
			// writeline
			case '\x9a' : 
			{
				// writeline
				if(ergebnis2 == '\xA0')
				{
					// line has been written!
				}
				else
				{
					errorcounter++;
					isyslog("alcd-plugin: ERROR: during linewrite\n");
				}
				break;
			}
			// set brightness
			case '\xf0' : 
			{
				// set brightness
				if(ergebnis2 == '\xA0')
				{
					// brightness could be set!
				}
				else
				{
					errorcounter++;
					isyslog("alcd-plugin: ERROR: during set brightness\n");
				}
				break;
			}
			// buttons
			case '\x92' : 
			{
				// read buttons 
				if(ergebnis2 == '\xA0')
				{
					// read buttons 2 bytes!

					// bit 0 reserved
					// bit 1 power button
					// bit 2 up button
					// bit 3 down button
					// bit 4 left button
					// bit 5 right button
					// bit 6 select button
					// bit 7 reserved

					
	                		read(fd,&ergebnis3,1);
					if((((((unsigned) ergebnis3) >> 4) & ~(~0 << 1)) && ((((unsigned) ergebnis3) >> 5) & ~(~0 << 1))))
					{
						//isyslog("alcd-plugin: menu button pressed\n");
						if(checkKeyTimer())
						{
							cRemote::Put(kMenu);
						}
					}
					else 
					if((((((unsigned) ergebnis3) >> 2) & ~(~0 << 1)) && ((((unsigned) ergebnis3) >> 3) & ~(~0 << 1))))
					{
						//isyslog("alcd-plugin: red button pressed\n");
						if(checkKeyTimer())
						{
							cRemote::Put(kRed);
						}
					}
					else 
					if((((((unsigned) ergebnis3) >> 2) & ~(~0 << 1)) && ((((unsigned) ergebnis3) >> 5) & ~(~0 << 1))))
					{
						//isyslog("alcd-plugin: blue button pressed\n");
						if(checkKeyTimer())
						{
							cRemote::Put(kBlue);
						}
					}
					else 
					if((((((unsigned) ergebnis3) >> 4) & ~(~0 << 1)) && ((((unsigned) ergebnis3) >> 3) & ~(~0 << 1))))
					{
						//isyslog("alcd-plugin: cd-eject button pressed\n");
						system("/bin/eject cdrom");
					}
					else 
					if(((((unsigned) ergebnis3) >> 1) & ~(~0 << 1)) && powerpressed == 0)
					{
						//isyslog("alcd-plugin: power button pressed\n");

						// get current time
						currenttime = time(NULL);

  						powerpressed = 1;
					}
					else
					if(((((unsigned) ergebnis3) >> 6) & ~(~0 << 1)) && powerpressed == 2)
					{
						//isyslog("alcd-plugin: shutdown confirmed (ok) pressed\n");

				  		write(fd,POWERLED_GREEN_BLINK,2);
				              	read(fd,&eventergebnis1,1);
				               	read(fd,&eventergebnis2,1);
				
						// led set?
						if((eventergebnis1 == '\x90') && (eventergebnis2 == '\xA0'))
						{
							// led has been set!
						}
						else
						{
							isyslog("alcd-plugin: ERROR: during led set\n");
						}

  						powerpressed = 0;
						system("/sbin/halt");
					}
					else
					if((((unsigned) ergebnis3) >> 2) & ~(~0 << 1))
					{
						//isyslog("alcd-plugin: up button pressed\n");
						if(checkKeyTimer())
						{
							cRemote::Put(kUp);
						}
					}
					else
					if((((unsigned) ergebnis3) >> 3) & ~(~0 << 1))
					{
						//isyslog("alcd-plugin: down button pressed\n");
						if(checkKeyTimer())
						{
							cRemote::Put(kDown);
						}
					}
					else
					if((((unsigned) ergebnis3) >> 4) & ~(~0 << 1))
					{
						//isyslog("alcd-plugin: left button pressed\n");
						if(checkKeyTimer())
						{
							cRemote::Put(kLeft);
						}
					}
					else
					if((((unsigned) ergebnis3) >> 5) & ~(~0 << 1))
					{
						//isyslog("alcd-plugin: right button pressed\n");
						if(checkKeyTimer())
						{
							cRemote::Put(kRight);
						}
					}
					else
					if((((unsigned) ergebnis3) >> 6) & ~(~0 << 1))
					{
						//isyslog("alcd-plugin: select button pressed\n");
						if(checkKeyTimer())
						{
							cRemote::Put(kOk);
						}
					}
					else
					{
						// unknown button statuscode
					}
				}
				else
				{
					errorcounter++;
					isyslog("alcd-plugin: ERROR: during read buttons\n");
				}
				break;
			}
			default :
			{
				// command unknown!
				isyslog("alcd-plugin: ERROR: unknown command-resultcode! Command (STRING): |%s| Byte1 (HEX): |%x| Byte2 (HEX): |%x|\n",command,ergebnis1,ergebnis2);
			}
		}

		// check STATUS return byte
		if(ergebnis2 != '\xA0')
		{
			errorcounter++;
	                isyslog("alcd-plugin: ERROR: Byte1 (HEX): |%x| Byte2 (HEX): |%x|\n",ergebnis1,ergebnis2);
			isyslog("alcd-plugin: ERROR: Performing PICRESET!\n");

               		read(fd,&ergebnis3,1);
	                read(fd,&ergebnis4,1);
	                read(fd,&ergebnis5,1);
	                read(fd,&ergebnis6,1);

	  		write(fd,PICRESET,2);
                	result = read(fd,&ergebnis3,1);
                	result = read(fd,&ergebnis4,1);
			if(ergebnis4 != '\xA0')
			{
		                isyslog("alcd-plugin: ERROR: Ergebniswert PICRESET: |%x|%c|\n",ergebnis4,ergebnis4);
			}
			usleep(100);
			system("/etc/init.d/activy.sh"); 	

			// Display 1. Line setzen
		  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE0, currentline1, '\x00');

			// send real command to lcd
		  	write(fd,output,23);
	                result = read(fd,&ergebnis1,1);
	                result = read(fd,&ergebnis2,1);

			// Display 2. Line setzen
		  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE1, currentline2,'\x00');

			// send real command to lcd
		  	write(fd,output,23);
	                result = read(fd,&ergebnis1,1);
	                result = read(fd,&ergebnis2,1);

		}

#ifdef D_EVENTFILTER
  		write(fd,EVENTFILTEROFF,2);
              	read(fd,&eventergebnis1,1);
               	read(fd,&eventergebnis2,1);

		// eventfilter set?
		if((eventergebnis1 == '\x95') && (eventergebnis2 == '\xA0'))
		{
			// eventfilter has been reset!
		}
		else
		{
			isyslog("alcd-plugin: ERROR: during eventfilter reset\n");
		}
#endif

		usleep(10000);

	 	//SQ: leave critical area
		//isyslog("alcd-plugin: leaving critical area");
	 	semop(semid, &leave, 1);
	 	
		locked = 0;
		// isyslog("alcd-plugin: finished sending to lcd: |%s|\n",command);
	}
}


/////////////////////////////////////////////////////////
// Check Timer Status

void activyLCD::checkTimer()
{
	time_t timertime;

	timertime = time(NULL);
	
	if(difftime(timertime,currenttime)>5)
	{
		powerpressed = 0;
		isyslog("alcd-plugin: Power-Timer > 5 Sec. Reset Power-Button.\n");
	}
}


/////////////////////////////////////////////////////////
// Check Errors

void activyLCD::checkErrors()
{
	time_t timertime;

	timertime = time(NULL);
	
	if(difftime(timertime,error_currenttime)>30)
	{
		if(errorcounter > 10)
		{
			isyslog("alcd-plugin: ERROR: too much errors - bailing out - stopping alcd-plugin!!\n");
			exit(0);
		}
		else
		{
			errorcounter = 0;

			// get current time
			error_currenttime = time(NULL);
		}
	}
}

/////////////////////////////////////////////////////////
// Check Keytimer

int activyLCD::checkKeyTimer()
{
	struct timeb *temptimer;
	temptimer = (struct timeb *) malloc(1);
	ftime(temptimer);

	// check timebase
	// only accept newly pressed button, if time is ok 
	if(keycounter == NULL)
	{
		// if keycounter == NULL, no keycounter is running!
//		isyslog("alcd-plugin: OK - set Key-Timer!\n");
		keycounter = (struct timeb *) malloc (1);
		ftime(keycounter);
		free(temptimer);
		return(1);
	}
	else
	{
		// if keycounter != NULL, check elapsed time
//		isyslog("alcd-plugin: elapsed milliseconds: 2: %i s %i ms 1: %i s %i ms diff: %i\n",temptimer->time,temptimer->millitm,keycounter->time,keycounter->millitm,((temptimer->time - keycounter->time)*1000+(temptimer->millitm - keycounter->millitm)));
		if(((temptimer->time - keycounter->time)*2000+(temptimer->millitm - keycounter->millitm)) > KEYCLICKTIMER)
		{
//			isyslog("alcd-plugin: Key-Timer has elapsed - freeing!\n");
			free(keycounter);
			keycounter = NULL;
			free(temptimer);
			return(0);
		}
		else
		{
			free(temptimer);
			return(0);
		}
	}
	free(temptimer);
	return(0);
}


/////////////////////////////////////////////////////////
// Return Power-Pressed-Status

int activyLCD::is_powerpressed()
{
  	return(powerpressed);
}


/////////////////////////////////////////////////////////
// Set Power-Pressed-Status

void activyLCD::set_powerpressed(int newpowerstatus)
{
  	powerpressed = newpowerstatus;
}


/////////////////////////////////////////////////////////
// Init LCD Device

void activyLCD::LCDInit()
{
//  	SetLEDBrightness(LEDBrightness);
}


/////////////////////////////////////////////////////////
// Read Buttons

void activyLCD::ReadButtons()
{
	char output[30];

	// Read Buttons Status
  	snprintf(output,2, "%s", READBUTTONS);
  	SendCommandToLCD(output,2 );
}


/////////////////////////////////////////////////////////
// Replace German Umlauts

char *activyLCD::ReplaceUmlaute(const char *line)
{
	// only proceed, if valid data
	if(line)
	{
	  	int b;  
	  	char c;
	  	char replacetemp1[strlen(line)*2];
		char *linetemp;
	        char replaced[strlen(line)*2];
	
		int size = strlen(line);
	
	  	replacetemp1[0]='\0'; 
	  	memset(replaced,' ',strlen(line)*2);
	
		linetemp = strdup(line);
	
	  	for (b=0; b<size; b++) 
		{
	  	  	c = linetemp[b];
	
	   	 	switch(c) 
			{
	     	 		case '\x09':
	      	  		strcat(replacetemp1," ");
	        		break; 
	
	    	 		case '':
	      	  		strcat(replacetemp1,"ae");
	        		break; 
	
	      			case '':
	        		strcat(replacetemp1,"oe");
	        		break; 
	
	      			case '':
	        		strcat(replacetemp1,"ue");
	        		break; 
	
	      			case '':
	        		strcat(replacetemp1,"ss");
	        		break; 
	
	      			case '':
	        		strcat(replacetemp1,"Ae");
	        		break; 
	
	      			case '':
	        		strcat(replacetemp1,"Oe");
	        		break; 
	
	      			case '':
	        		strcat(replacetemp1,"Ue");
	        		break;
	
	      			default:
	        		replacetemp1[strlen(replacetemp1) + 1]=0;
	        		replacetemp1[strlen(replacetemp1)] = c;
	    		};
	  	};
	  	replacetemp1[strlen(replacetemp1)+1]='\0';
	
		return(strdup(replacetemp1));
	}
	else
	{
		// return error message
		return(strdup("-- Keine Daten --"));
	}
}


/////////////////////////////////////////////////////////
// Set LCD-Lines on LCD

void activyLCD::SetLCDLines(char text1[21],  char text2[21])
{
  	LCDInit();

  	char text[21];
	char output[30];

  	snprintf(currentline1,21,"%-20s",text1);
  	snprintf(currentline2,21,"%-20s",text2);

	// 1. Line zusammensetzen
  	memset(text,' ',21);
  	snprintf(text,21,"%-20s",text1);

	// Display 1. Line clearen
  	snprintf(output,3, "%s%c", LCD_WRITE_TEXT_LINE0, '\x00');
  	SendCommandToLCD(output,3 );

	// Display 1. Line setzen
  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE0, text, '\x00');
  	SendCommandToLCD(output,23 );

	// 2. Line zusammensetzen
  	memset(text,' ',21);
  	snprintf(text,21,"%-20s",text2);

	// Display 2. Line clearen
 	snprintf(output,3, "%s%c", LCD_WRITE_TEXT_LINE1, '\x00');
  	SendCommandToLCD(output,3 );

	// Display 2. Line setzen
  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE1, text,'\x00');
  	SendCommandToLCD(output,23 );

}

/////////////////////////////////////////////////////////
// Set LCD-Line0 on LCD

void activyLCD::SetLCDLine0(char *text1)
{
  	char text[21];
	char output[30];

	// only proceed, if valid data
	if(text1)
	{
  		snprintf(currentline1,21,"%-20s",text1);

	  	LCDInit();
	
		// assemble line 0
	  	memset(text,' ',21);
	  	snprintf(text,21,"%-20s",ReplaceUmlaute(text1));

		// set activy display line 0
	  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE0, text, '\x00');
	  	SendCommandToLCD(output,23 );
	}
	else
	{
		// no valid data received
		// set activy display line 0
	  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE0, "-- Keine Daten --", '\x00');
	  	SendCommandToLCD(output,23 );
	}
}

/////////////////////////////////////////////////////////
// Set LCD-Line1 on LCD

void activyLCD::SetLCDLine1(char *text1)
{
    	char text[21];
	char output[30];

	// only proceed, if valid data
	if(text1)
	{
  		snprintf(currentline2,21,"%-20s",text1);

		LCDInit();

		// assemble line 1
	  	memset(text,' ',21);
	  	snprintf(text,21,"%-20s",ReplaceUmlaute(text1));

		// set activy display line 1
	  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE1, text, '\x00');
	  	SendCommandToLCD(output,23 );
	}
	else
	{
		// no valid data received
		// set activy display line 1
	  	snprintf(output,23, "%s%s%c", LCD_WRITE_TEXT_LINE1, "-- Keine Daten --", '\x00');
	  	SendCommandToLCD(output,23 );
	}
}

/////////////////////////////////////////////////////////
// Clear LCD Device

void activyLCD::LCDClear() // clears the lcd display
{
	char output[30];

  	snprintf(output,2, "%s%c", LCD_CLEAR, '\x00');
  	SendCommandToLCD(output,2 );
}

/////////////////////////////////////////////////////////
// Set Power Button

void activyLCD::SetPowerButton(bool status) // switch powerbutton on
{
	char output[30];

  	switch (status) 
	{
    		case 0:
      		snprintf(output,2, "%s", POWERBUTTON_SCANCODE_OFF);
		SendCommandToLCD(output,2 );
      		snprintf(output,2, "%s", POWERBUTTON_OFF);
  		SendCommandToLCD(output,2 );
      		break;

    		case 1:
      		snprintf(output,2, "%s", POWERBUTTON_SCANCODE_ON);
  		SendCommandToLCD(output,2 );
      		snprintf(output,2, "%s", POWERBUTTON_ON);
		SendCommandToLCD(output,2 );
      		break;
  	}
}

/////////////////////////////////////////////////////////
// Set LED Brightness

void activyLCD::SetLEDBrightness(const int brightness)
{
 	int brightness_temp = 0;
	char output[30];

  	switch(brightness) 
	{ 
    		case 0: // superbright
      		brightness_temp = 0;
      		break;

    		case 1: // bright
      		brightness_temp = 50;
      		break;

    		case 2: // semibright
      		brightness_temp = 100;
      		break;

    		case 3: // dark
      		brightness_temp = 150;
      		break;

    		case 4: // off
      		brightness_temp = 255;
      		break;
	}

  	snprintf(output,2, "%s%c", LCD_LEDBRIGHTNESS, brightness_temp);
 	SendCommandToLCD(output,2 );
}

#endif
